/*
 Includes for gps-clock

 Information Memory (User Flash)  0x01000 - 0x10FF  256 bytes as 4 x 64 byte segments
   0x01000 - 0x0103F   INFOD
   0x01040 - 0x0107F   INFOC
   0x01080 - 0x010BF   INFOB
   0x010C0 - 0x010FF   INFOA
*/

const char *INFOC = (char*)0x1040;   //INFO C
void erase_SegC(void);
void write_SegC(char, char, char);
void write_SegC_test(char, char, char);  // test it so we don't wear out an actual part
signed char wr_flash_flag = 0;

#define TXD BIT1        // UART for NEO6M GPS module
#define RXD BIT2

// Messages
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

// Declare the messages
const struct messages msg1 =  {0x80, 16, "NEO6M GPS Clock "};
const struct messages msg2 =  {0xC0, 16, " Jed Margolin   "};
const struct messages msg3 =  {0x8B, 5,  "Local"};
const struct messages msg4 =  {0x8B, 6,  " UTC  "};
const struct messages msg5 =  {0x80, 10, "UTC Offset"};
const struct messages msg6 =  {0x80, 11, "Local 12/24"};
const struct messages msg7 =  {0x8D, 2,  "12"};
const struct messages msg8 =  {0x8D, 2,  "24"};
const struct messages msg9 =  {0x80, 3,  "DST"};
const struct messages msg10 = {0x84, 12,  "Observed    "};
const struct messages msg11 = {0x84, 12,  "Not Observed"};
const struct messages msg12 = {0x8C, 5,  "AM   "};
const struct messages msg13 = {0x8C, 5,  "PM   "};
const struct messages msg17 = {0x80, 12,  "DST Start 3/"};
const struct messages msg18 = {0xC0, 12,  "DST End  11/"};
const struct messages msg19 =  {0x80, 14, "# Sats in View"};
const struct messages msg20 =  {0x81, 14, "GPS Alarm Mode"};
const struct messages msg21 =  {0xCA, 5, " Good "};
const struct messages msg22 =  {0xCA, 5, "  Ok "};
const struct messages msg23 =  {0xCA, 5, "Oh Oh"};
const struct messages msg24 =  {0x84, 8, "Software"};
const struct messages msg25 =  {0xC2, 11, "Version 1.0"};
const struct messages msg26 =  {0x80, 15, "Waiting For PPS"};

void domsg(const struct messages *);

// Messages for the days of the week
const char day_array[7][3] = {"Sat","Sun","Mon","Tue","Wed","Thu","Fri"};

const unsigned char GPSHEADER1[6] = {"GNZDA,"};  // GPS Header $GNZDA; it seems to prefer this to GPZDA
const unsigned char GPSHEADER2[6] = {"GPGSV,"};  // GPS Header $GPGSV

unsigned char gps_buffer[32];
unsigned char checksum[2];
unsigned char gotit=0, data_count=0, comm_error=0;

void get_gps_msg1(void);  // $GNZDA
void get_gps_msg2(void);  // $GPGSV
unsigned char gps_msg_number;
void decode_data(void);
unsigned char satinv[2];

// For UTC time
void convert_utc_ascii(void);
void convert_utc_bin(void);

unsigned char utc_hour[2];
unsigned char *utc_hour_ptr = utc_hour;

unsigned char utc_min[2];
unsigned char *utc_min_ptr = utc_min;

unsigned char utc_sec[2];
unsigned char *utc_sec_ptr = utc_sec;

unsigned char utc_day[2];
unsigned char *utc_day_ptr = utc_day;

unsigned char utc_month[2];
unsigned char *utc_mon_ptr = utc_month;

unsigned char utc_year[4];
unsigned char *utc_year_ptr = utc_year;

signed char utc_offset;

unsigned char last_utc_sec[2];

signed int ut_year, ut_month, ut_day, ut_hour, ut_minute, ut_second;

// For Local Time
signed int local_year, local_month, local_day, local_hour, local_minute, local_second;
unsigned char leapyear;

unsigned char loc_second[2];
unsigned char *loc_sec_ptr = loc_second;

unsigned char loc_minute[2];
unsigned char *loc_min_ptr = loc_minute;

unsigned char loc_hour[2];
unsigned char *loc_hour_ptr = loc_hour;

unsigned char loc_day[2];
unsigned char *loc_day_ptr = loc_day;

unsigned char loc_month[2];
unsigned char *loc_month_ptr = loc_month;

unsigned char loc_year[4];
unsigned char *loc_year_ptr = loc_year;

unsigned char loc_offset[4];
unsigned char *loc_offset_ptr = loc_offset;

signed char day, LastDay;
signed int save_year;
signed char time_offset;    // local time offset
char dst_hour = 0;
char hours12 = 1;
unsigned char pmflag, sats_in_view;

// Days in the months - non-leap year
const signed char months[13] =
{ 0,    // unused
  31,   // January
  28,   // February
  31,   // March
  30,   // April
  31,   // May
  30,   // June
  31,   // July
  31,   // August
  30,   // September
  31,   // October
  30,   // November
  31    // December
};

// Days in the months - leap year
const signed char months_ly[13] =
{ 0,    // unused
  31,   // January
  29,   // February
  31,   // March
  30,   // April
  31,   // May
  30,   // June
  31,   // July
  31,   // August
  30,   // September
  31,   // October
  30,   // November
  31    // December
};

// For reading, debouncing, and processing the switches
// We only need to know if a switch has been pressed
struct myswitch
{
    unsigned char debounce;     // debounce byte
    unsigned char state;        // debounced state of the switch
    unsigned char old;          // old state
    unsigned char press;        // switch has been pressed
};

// Initialize switches
struct myswitch switch1 = {0xFF,0x01,0x01,0x00};
struct myswitch switch2 = {0xFF,0x01,0x01,0x00};
struct myswitch switch3 = {0xFF,0x01,0x01,0x00};

void do_switch(unsigned char, unsigned char, struct myswitch *);   // PortIn, Bit, switch structure
//--------------------------------------------------------
unsigned char is_leap(void);
unsigned char is_leap2(unsigned int);
unsigned char get_day_utc(void);
unsigned char get_day_local(void);
void decode_time(void);
void get_local_minus(signed char);
void get_local_plus(signed char);
void time12(void);
unsigned int timeout;
unsigned char sound = 1;
unsigned char sound_enable = 0;
void do_sounds(void);
//===================================================
void find_dst(unsigned int);         // find the start and end dates for DST
unsigned char is_dst(void);                   // Are we in the DST period?
unsigned char get_day(int,int,int);  // get teh day of the week
unsigned char dst_window;    // 1 = we are in the dst window
unsigned char dst_start_date, dst_end_date;
unsigned char dst_observed;   // 1 = observe DST

const unsigned int monthkey[12] = {1,4,4,0,2,5,0,3,6,1,4,6};  // month's key value
const unsigned char dst_start_table[7] = {9,8,14,13,12,11,10};
unsigned char dst_end_table[7] = {2,1,7,6,5,4,3};

void waitsec(void);   // Delay for Greeting message

volatile unsigned int timer_flag = 10, timer_count = 250;
volatile unsigned char display = 0;
volatile long temp,temp1,temp2;
unsigned char state=0, count, switch_time = 0;
unsigned char rxbuffer[64];   // buffer for GPS message
unsigned char oldsec1, newsec1, newsec2, oldsec2, blink;
unsigned char my_second_flag = 0;
//--------------------------------------------------------
// For LCD
void display_utc(void);    // Display UTC
void display_local();    // Display local time

unsigned char convert_hex_bcd2(signed char, unsigned char *);   // binary value, & unsigned output string
void convert_hex_bcd4(unsigned int, unsigned char *);   // binary value, & unsigned output string

void lcd_init(void);
void lcd_reset(void);
void wait10ms(void);
void lcd_wr4(unsigned int);
void lcd_wr_instr(unsigned int);
void lcd_wr_data(unsigned int);
void lcd_wr_bcd(unsigned int, unsigned int);  // position, value
void lcd_wr_bcd_long(unsigned int, unsigned long);
void clear_lcd(unsigned char);   // one line
//===================================================
/*
// For preliminary testing

struct utc_time
{
   unsigned char hour[2];
   unsigned char min[2];
   unsigned char sec[2];
   unsigned char month[2];
   unsigned char day[2];
   unsigned char year[4];
   signed char offset;
};

const struct utc_time test_array[17] =
{
   {"13","55","37","12","05","2020",-8},  // 0 - same day
   {"02","46","12","12","06","2020",-8},  // 1 - previous day
   {"02","46","15","12","01","2020",-8},  // 2 - previous day and previous month
   {"02","34","23","01","01","2021",-8},  // 3 - previous day, month, and year
   {"20","23","34","12","05","2020",-8},  // 4 - noon
   {"08","10","45","06","08","2020",-8},  // 5 - midnight

   // For positive offset
   {"13","55","37","12","05","2020",8},   // 6 - same day
   {"20","46","12","12","08","2020",8},   // 7 - next day
   {"21","46","15","11","30","2020",8},   // 8 - next day and next month
   {"22","34","23","12","31","2020",8},   // 9 - next day, month, and yea

   // Test the days of the week
   {"06","00","00","12","07","2020",0},   // 10
   {"06","00","00","12","08","2020",0},   // 11
   {"06","00","00","12","09","2020",0},   // 12
   {"06","00","00","12","10","2020",0},   // 13
   {"06","00","00","12","11","2020",0},   // 14
   {"06","00","00","12","12","2020",0},   // 15
   {"06","00","00","12","13","2020",0}   // 16
};
*/
//=================================================
void state0(void), state1(void), state2(void), state3(void), state4(void), state5(void), state6(void);
void state7(void), state8(void), state9(void), state10(void), state11(void), state12(void), state13(void);
void state14(void), state15(void), state16(void), state17(void), state18(void), state19(void), state20(void);
void state21(void);

void (*state_ptr[22])() = {state0,state1,state2,state3,state4,state5,state6,state7,state8,state9,
state10,state11,state12,state13,state14,state15,state16,state17,state18,state19,state20,state21};
//=================================================


